@@ -80,7 +80,9 @@ enum class Command : uint32_t {
     ZOOM_OUT,
     ZOOM_TO_FIT,
     SHOW_GRID,
+    DIM_SOLID_MODEL,
     PERSPECTIVE_PROJ,
+    EXPLODE_SKETCH,
     ONTO_WORKPLANE,
     NEAREST_ORTHO,
     NEAREST_ISO,
@@ -88,6 +90,7 @@ enum class Command : uint32_t {
     SHOW_TOOLBAR,
     SHOW_TEXT_WND,
     UNITS_INCHES,
+    UNITS_FEET_INCHES,
     UNITS_MM,
     UNITS_METERS,
     FULL_SCREEN,
@@ -179,7 +182,7 @@ public:
     enum {
         MAX_COLS = 100,
         MIN_COLS = 45,
-        MAX_ROWS = 2000
+        MAX_ROWS = 4000
     };
 
     typedef struct {
@@ -311,12 +314,14 @@ public:
         EXPORT_OFFSET         = 110,
         CANVAS_SIZE           = 111,
         G_CODE_DEPTH          = 112,
-        G_CODE_PASSES         = 113,
-        G_CODE_FEED           = 114,
-        G_CODE_PLUNGE_FEED    = 115,
-        AUTOSAVE_INTERVAL     = 116,
-        LIGHT_AMBIENT         = 117,
-        FIND_CONSTRAINT_TIMEOUT = 118,
+        G_CODE_SAFE_HEIGHT    = 113,
+        G_CODE_PASSES         = 114,
+        G_CODE_FEED           = 115,
+        G_CODE_PLUNGE_FEED    = 116,
+        AUTOSAVE_INTERVAL     = 117,
+        LIGHT_AMBIENT         = 118,
+        FIND_CONSTRAINT_TIMEOUT = 119,
+        EXPLODE_DISTANCE      = 120,
         // For TTF text
         TTF_TEXT              = 300,
         // For the step dimension screen
@@ -341,7 +346,9 @@ public:
         VIEW_PROJ_RIGHT       = 702,
         VIEW_PROJ_UP          = 703,
         // For tangent arc
-        TANGENT_ARC_RADIUS    = 800
+        TANGENT_ARC_RADIUS    = 800,
+        // For helix pitch
+        HELIX_PITCH           = 802
     };
     struct {
         bool        showAgain;
@@ -472,6 +479,8 @@ public:
     static void ScreenChangeExprA(int link, uint32_t v);
     static void ScreenChangeGroupName(int link, uint32_t v);
     static void ScreenChangeGroupScale(int link, uint32_t v);
+    static void ScreenChangeHelixPitch(int link, uint32_t v);
+    static void ScreenChangePitchOption(int link, uint32_t v);
     static void ScreenChangeLightDirection(int link, uint32_t v);
     static void ScreenChangeLightIntensity(int link, uint32_t v);
     static void ScreenChangeLightAmbient(int link, uint32_t v);
@@ -482,6 +491,7 @@ public:
     static void ScreenChangeExportMaxSegments(int link, uint32_t v);
     static void ScreenChangeCameraTangent(int link, uint32_t v);
     static void ScreenChangeGridSpacing(int link, uint32_t v);
+    static void ScreenChangeExplodeDistance(int link, uint32_t v);
     static void ScreenChangeDigitsAfterDecimal(int link, uint32_t v);
     static void ScreenChangeDigitsAfterDecimalDegree(int link, uint32_t v);
     static void ScreenChangeUseSIPrefixes(int link, uint32_t v);
@@ -532,7 +542,9 @@ public:
     Platform::MenuRef linkRecentMenu;
 
     Platform::MenuItemRef showGridMenuItem;
+    Platform::MenuItemRef dimSolidModelMenuItem;
     Platform::MenuItemRef perspectiveProjMenuItem;
+    Platform::MenuItemRef explodeMenuItem;
     Platform::MenuItemRef showToolbarMenuItem;
     Platform::MenuItemRef showTextWndMenuItem;
     Platform::MenuItemRef fullScreenMenuItem;
@@ -540,6 +552,7 @@ public:
     Platform::MenuItemRef unitsMmMenuItem;
     Platform::MenuItemRef unitsMetersMenuItem;
     Platform::MenuItemRef unitsInchesMenuItem;
+    Platform::MenuItemRef unitsFeetInchesMenuItem;
 
     Platform::MenuItemRef inWorkplaneMenuItem;
     Platform::MenuItemRef in3dMenuItem;
@@ -727,6 +740,7 @@ public:
     public:
         int         zIndex;
         double      distance;
+        double      depth;
         Selection   selection;
     };
 
@@ -795,6 +809,8 @@ public:
     bool    showEdges;
     bool    showOutlines;
     bool    showFaces;
+    bool    showFacesDrawing;
+    bool    showFacesNonDrawing;
     bool    showMesh;
     void ToggleBool(bool *v);
 
@@ -802,6 +818,7 @@ public:
     DrawOccludedAs drawOccludedAs;
 
     bool    showSnapGrid;
+    bool    dimSolidModel;
     void DrawSnapGrid(Canvas *canvas);
 
     void AddPointToDraggedList(hEntity hp);
@@ -824,7 +841,7 @@ public:
     void MouseLeftDoubleClick(double x, double y);
     void MouseMiddleOrRightDown(double x, double y);
     void MouseRightUp(double x, double y);
-    void MouseScroll(double x, double y, int delta);
+    void MouseScroll(double x, double y, double delta);
     void MouseLeave();
     bool KeyboardEvent(Platform::KeyboardEvent event);
     void EditControlDone(const std::string &s);
